@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack.compute.v2 import aggregate as _aggregate
 from openstack.compute.v2 import availability_zone
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor as _flavor
@@ -23,11 +24,11 @@ from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import service as _service
 from openstack.compute.v2 import volume_attachment as _volume_attachment
-from openstack import proxy2
-from openstack import resource2
+from openstack import proxy
+from openstack import resource
 
 
-class Proxy(proxy2.BaseProxy):
+class Proxy(proxy.Proxy):
 
     def find_extension(self, name_or_id, ignore_missing=True):
         """Find a single extension
@@ -50,7 +51,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: A generator of extension instances.
         :rtype: :class:`~openstack.compute.v2.extension.Extension`
         """
-        return self._list(extension.Extension, paginated=False)
+        return self._list(extension.Extension, paginated=True)
 
     def find_flavor(self, name_or_id, ignore_missing=True):
         """Find a single flavor
@@ -112,7 +113,7 @@ class Proxy(proxy2.BaseProxy):
             :class:`~openstack.compute.v2.flavor.FlavorDetail` objects,
             otherwise :class:`~openstack.compute.v2.flavor.Flavor`.
             *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the flavors being returned.
 
         :returns: A generator of flavor objects
@@ -120,6 +121,72 @@ class Proxy(proxy2.BaseProxy):
         flv = _flavor.FlavorDetail if details else _flavor.Flavor
         return self._list(flv, paginated=True, **query)
 
+    def aggregates(self):
+        """Return a generator of aggregate
+
+        :returns: A generator of aggregate
+        :rtype: class: `~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = _aggregate.Aggregate
+
+        return self._list(aggregate, paginated=False)
+
+    def get_aggregate(self, aggregate):
+        """Get a single host aggregate
+
+        :param image: The value can be the ID of an aggregate or a
+                      :class:`~openstack.compute.v2.aggregate.Aggregate`
+                      instance.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_aggregate.Aggregate, aggregate)
+
+    def create_aggregate(self, **attrs):
+        """Create a new host aggregate from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create a
+                           :class:`~openstack.compute.v2.aggregate.Aggregate`,
+                           comprised of the properties on the Aggregate class.
+
+        :returns: The results of aggregate creation
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._create(_aggregate.Aggregate, **attrs)
+
+    def update_aggregate(self, aggregate, **attrs):
+        """Update a host aggregate
+
+        :param server: Either the ID of a host aggregate or a
+                       :class:`~openstack.compute.v2.aggregate.Aggregate`
+                       instance.
+        :attrs kwargs: The attributes to update on the aggregate represented
+                       by ``aggregate``.
+
+        :returns: The updated aggregate
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._update(_aggregate.Aggregate, aggregate, **attrs)
+
+    def delete_aggregate(self, aggregate, ignore_missing=True):
+        """Delete a host aggregate
+
+        :param keypair: The value can be either the ID of an aggregate or a
+                        :class:`~openstack.compute.v2.aggregate.Aggregate`
+                        instance.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the aggregate does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent aggregate.
+
+        :returns: ``None``
+        """
+        self._delete(_aggregate.Aggregate, aggregate,
+                     ignore_missing=ignore_missing)
+
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
 
@@ -168,7 +235,7 @@ class Proxy(proxy2.BaseProxy):
             :class:`~openstack.compute.v2.image.ImageDetail` objects,
             otherwise :class:`~openstack.compute.v2.image.Image`.
             *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of image objects
@@ -198,7 +265,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.image.Image`
         """
         res = self._get_base_resource(image, _image.Image)
-        metadata = res.get_metadata(self._session)
+        metadata = res.get_metadata(self)
         result = _image.Image.existing(id=res.id, metadata=metadata)
         return result
 
@@ -219,7 +286,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.image.Image`
         """
         res = self._get_base_resource(image, _image.Image)
-        metadata = res.set_metadata(self._session, **metadata)
+        metadata = res.set_metadata(self, **metadata)
         result = _image.Image.existing(id=res.id, metadata=metadata)
         return result
 
@@ -237,7 +304,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: ``None``
         """
         res = self._get_base_resource(image, _image.Image)
-        return res.delete_metadata(self._session, keys)
+        return res.delete_metadata(self, keys)
 
     def create_keypair(self, **attrs):
         """Create a new keypair from attributes
@@ -335,13 +402,13 @@ class Proxy(proxy2.BaseProxy):
                     When set to ``True``, no exception will be set when
                     attempting to delete a nonexistent server
         :param bool force: When set to ``True``, the server deletion will be
-                           forced immediatly.
+                           forced immediately.
 
         :returns: ``None``
         """
         if force:
             server = self._get_resource(_server.Server, server)
-            server.force_delete(self._session)
+            server.force_delete(self)
         else:
             self._delete(_server.Server, server, ignore_missing=ignore_missing)
 
@@ -371,7 +438,7 @@ class Proxy(proxy2.BaseProxy):
         """
         return self._get(_server.Server, server)
 
-    def servers(self, details=True, **query):
+    def servers(self, details=True, all_projects=False, **query):
         """Retrieve a generator of servers
 
         :param bool details: When set to ``False``
@@ -379,7 +446,7 @@ class Proxy(proxy2.BaseProxy):
                     will be returned. The default, ``True``, will cause
                     :class:`~openstack.compute.v2.server.ServerDetail`
                     instances to be returned.
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
             the servers being returned.  Available parameters include:
 
             * changes_since: A time/date stamp for when the server last changed
@@ -395,6 +462,8 @@ class Proxy(proxy2.BaseProxy):
             * status: Value of the status of the server so that you can filter
                     on "ACTIVE" for example.
             * host: Name of the host as a string.
+            * all_projects: Flag to request servers be returned from all
+                            projects, not just the currently scoped one.
             * limit: Requests a specified page size of returned items from the
                      query.  Returns a number of items up to the specified
                      limit value. Use the limit parameter to make an initial
@@ -408,6 +477,8 @@ class Proxy(proxy2.BaseProxy):
 
         :returns: A generator of server instances.
         """
+        if all_projects:
+            query['all_projects'] = True
         srv = _server.ServerDetail if details else _server.Server
         return self._list(srv, paginated=True, **query)
 
@@ -434,7 +505,18 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.change_password(self._session, new_password)
+        server.change_password(self, new_password)
+
+    def get_server_password(self, server):
+        """Get the administrator password
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: encrypted password.
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.get_password(self._session)
 
     def reset_server_state(self, server, state):
         """Reset the state of server
@@ -447,7 +529,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         res = self._get_base_resource(server, _server.Server)
-        res.reset_state(self._session, state)
+        res.reset_state(self, state)
 
     def reboot_server(self, server, reboot_type):
         """Reboot a server
@@ -460,7 +542,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.reboot(self._session, reboot_type)
+        server.reboot(self, reboot_type)
 
     def rebuild_server(self, server, name, admin_password, **attrs):
         """Rebuild a server
@@ -488,7 +570,7 @@ class Proxy(proxy2.BaseProxy):
                   instance.
         """
         server = self._get_resource(_server.Server, server)
-        return server.rebuild(self._session, name, admin_password, **attrs)
+        return server.rebuild(self, name, admin_password, **attrs)
 
     def resize_server(self, server, flavor):
         """Resize a server
@@ -501,8 +583,8 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        flavor_id = resource2.Resource._get_id(flavor)
-        server.resize(self._session, flavor_id)
+        flavor_id = resource.Resource._get_id(flavor)
+        server.resize(self, flavor_id)
 
     def confirm_server_resize(self, server):
         """Confirm a server resize
@@ -513,7 +595,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.confirm_resize(self._session)
+        server.confirm_resize(self)
 
     def revert_server_resize(self, server):
         """Revert a server resize
@@ -524,7 +606,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.revert_resize(self._session)
+        server.revert_resize(self)
 
     def create_server_image(self, server, name, metadata=None):
         """Create an image from a server
@@ -537,7 +619,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.create_image(self._session, name, metadata)
+        server.create_image(self, name, metadata)
 
     def add_security_group_to_server(self, server, security_group):
         """Add a security group to a server
@@ -551,8 +633,8 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource2.Resource._get_id(security_group)
-        server.add_security_group(self._session, security_group_id)
+        security_group_id = resource.Resource._get_id(security_group)
+        server.add_security_group(self, security_group_id)
 
     def remove_security_group_from_server(self, server, security_group):
         """Remove a security group from a server
@@ -566,8 +648,8 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource2.Resource._get_id(security_group)
-        server.remove_security_group(self._session, security_group_id)
+        security_group_id = resource.Resource._get_id(security_group)
+        server.remove_security_group(self, security_group_id)
 
     def add_fixed_ip_to_server(self, server, network_id):
         """Adds a fixed IP address to a server instance.
@@ -579,7 +661,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.add_fixed_ip(self._session, network_id)
+        server.add_fixed_ip(self, network_id)
 
     def remove_fixed_ip_from_server(self, server, address):
         """Removes a fixed IP address from a server instance.
@@ -591,7 +673,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.remove_fixed_ip(self._session, address)
+        server.remove_fixed_ip(self, address)
 
     def add_floating_ip_to_server(self, server, address, fixed_address=None):
         """Adds a floating IP address to a server instance.
@@ -605,7 +687,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.add_floating_ip(self._session, address,
+        server.add_floating_ip(self, address,
                                fixed_address=fixed_address)
 
     def remove_floating_ip_from_server(self, server, address):
@@ -618,7 +700,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.remove_floating_ip(self._session, address)
+        server.remove_floating_ip(self, address)
 
     def backup_server(self, server, name, backup_type, rotation):
         """Backup a server
@@ -634,7 +716,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.backup(self._session, name, backup_type, rotation)
+        server.backup(self, name, backup_type, rotation)
 
     def pause_server(self, server):
         """Pauses a server and changes its status to ``PAUSED``.
@@ -644,7 +726,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.pause(self._session)
+        server.pause(self)
 
     def unpause_server(self, server):
         """Unpauses a paused server and changes its status to ``ACTIVE``.
@@ -654,7 +736,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unpause(self._session)
+        server.unpause(self)
 
     def suspend_server(self, server):
         """Suspends a server and changes its status to ``SUSPENDED``.
@@ -664,7 +746,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.suspend(self._session)
+        server.suspend(self)
 
     def resume_server(self, server):
         """Resumes a suspended server and changes its status to ``ACTIVE``.
@@ -674,7 +756,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.resume(self._session)
+        server.resume(self)
 
     def lock_server(self, server):
         """Locks a server.
@@ -684,7 +766,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.lock(self._session)
+        server.lock(self)
 
     def unlock_server(self, server):
         """Unlocks a locked server.
@@ -694,7 +776,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unlock(self._session)
+        server.unlock(self)
 
     def rescue_server(self, server, admin_pass=None, image_ref=None):
         """Puts a server in rescue mode and changes it status to ``RESCUE``.
@@ -711,7 +793,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.rescue(self._session, admin_pass=admin_pass,
+        server.rescue(self, admin_pass=admin_pass,
                       image_ref=image_ref)
 
     def unrescue_server(self, server):
@@ -722,7 +804,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unrescue(self._session)
+        server.unrescue(self)
 
     def evacuate_server(self, server, host=None, admin_pass=None, force=None):
         """Evacuates a server from a failed host to a new host.
@@ -739,7 +821,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.evacuate(self._session, host=host, admin_pass=admin_pass,
+        server.evacuate(self, host=host, admin_pass=admin_pass,
                         force=force)
 
     def start_server(self, server):
@@ -750,7 +832,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.start(self._session)
+        server.start(self)
 
     def stop_server(self, server):
         """Stops a running server and changes its state to ``SHUTOFF``.
@@ -760,7 +842,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.stop(self._session)
+        server.stop(self)
 
     def shelve_server(self, server):
         """Shelves a server.
@@ -775,7 +857,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.shelve(self._session)
+        server.shelve(self)
 
     def unshelve_server(self, server):
         """Unselves or restores a shelved server.
@@ -789,7 +871,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unshelve(self._session)
+        server.unshelve(self)
 
     def get_server_console_output(self, server, length=None):
         """Return the console output for a server.
@@ -802,12 +884,33 @@ class Proxy(proxy2.BaseProxy):
                     escaped to create a valid JSON string.
         """
         server = self._get_resource(_server.Server, server)
-        return server.get_console_output(self._session, length=length)
+        return server.get_console_output(self, length=length)
 
-    def wait_for_server(self, server, status='ACTIVE', failures=['ERROR'],
+    def wait_for_server(self, server, status='ACTIVE', failures=None,
                         interval=2, wait=120):
-        return resource2.wait_for_status(self._session, server, status,
-                                         failures, interval, wait)
+        """Wait for a server to be in a particular status.
+
+        :param res: The resource to wait on to reach the specified status.
+                    The resource must have a ``status`` attribute.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param status: Desired status.
+        :param failures: Statuses that would be interpreted as failures.
+        :type failures: :py:class:`list`
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to the desired status failed to occur in specified seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+                 has transited to one of the failure statuses.
+        :raises: :class:`~AttributeError` if the resource does not have a
+                ``status`` attribute.
+        """
+        failures = ['ERROR'] if failures is None else failures
+        return resource.wait_for_status(
+            self, server, status, failures, interval, wait)
 
     def create_server_interface(self, server, **attrs):
         """Create a new server interface from attributes
@@ -822,7 +925,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: The results of server interface creation
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._create(_server_interface.ServerInterface,
                             server_id=server_id, **attrs)
 
@@ -848,7 +951,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(server_interface, server,
                                             "server_id")
-        server_interface = resource2.Resource._get_id(server_interface)
+        server_interface = resource.Resource._get_id(server_interface)
 
         self._delete(_server_interface.ServerInterface,
                      port_id=server_interface,
@@ -874,7 +977,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(server_interface, server,
                                             "server_id")
-        server_interface = resource2.Resource._get_id(server_interface)
+        server_interface = resource.Resource._get_id(server_interface)
 
         return self._get(_server_interface.ServerInterface,
                          server_id=server_id, port_id=server_interface)
@@ -888,7 +991,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: A generator of ServerInterface objects
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._list(_server_interface.ServerInterface, paginated=False,
                           server_id=server_id)
 
@@ -903,7 +1006,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: A generator of ServerIP objects
         :rtype: :class:`~openstack.compute.v2.server_ip.ServerIP`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._list(server_ip.ServerIP, paginated=False,
                           server_id=server_id, network_label=network_label)
 
@@ -938,7 +1041,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.server.Server`
         """
         res = self._get_base_resource(server, _server.Server)
-        metadata = res.get_metadata(self._session)
+        metadata = res.get_metadata(self)
         result = _server.Server.existing(id=res.id, metadata=metadata)
         return result
 
@@ -959,7 +1062,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.server.Server`
         """
         res = self._get_base_resource(server, _server.Server)
-        metadata = res.set_metadata(self._session, **metadata)
+        metadata = res.set_metadata(self, **metadata)
         result = _server.Server.existing(id=res.id, metadata=metadata)
         return result
 
@@ -977,7 +1080,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: ``None``
         """
         res = self._get_base_resource(server, _server.Server)
-        return res.delete_metadata(self._session, keys)
+        return res.delete_metadata(self, keys)
 
     def create_server_group(self, **attrs):
         """Create a new server group from attributes
@@ -1041,7 +1144,7 @@ class Proxy(proxy2.BaseProxy):
     def server_groups(self, **query):
         """Return a generator of server groups
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of ServerGroup objects
@@ -1049,14 +1152,23 @@ class Proxy(proxy2.BaseProxy):
         """
         return self._list(_server_group.ServerGroup, paginated=False, **query)
 
-    def hypervisors(self):
+    def hypervisors(self, details=False):
         """Return a generator of hypervisor
 
+        :param bool details: When set to the default, ``False``,
+                    :class:`~openstack.compute.v2.hypervisor.Hypervisor`
+                    instances will be returned. ``True`` will cause
+                    :class:`~openstack.compute.v2.hypervisor.HypervisorDetail`
+                    instances to be returned.
         :returns: A generator of hypervisor
         :rtype: class: `~openstack.compute.v2.hypervisor.Hypervisor`
         """
+        if details:
+            hypervisor = _hypervisor.HypervisorDetail
+        else:
+            hypervisor = _hypervisor.Hypervisor
 
-        return self._list(_hypervisor.Hypervisor, paginated=False)
+        return self._list(hypervisor, paginated=False)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
@@ -1096,7 +1208,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         service = self._get_resource(_service.Service, service)
-        service.force_down(self._session, host, binary)
+        service.force_down(self, host, binary)
 
     def disable_service(self, service, host, binary, disabled_reason=None):
         """Disable a service
@@ -1110,7 +1222,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         service = self._get_resource(_service.Service, service)
-        service.disable(self._session,
+        service.disable(self,
                         host, binary,
                         disabled_reason)
 
@@ -1126,7 +1238,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         service = self._get_resource(_service.Service, service)
-        service.enable(self._session, host, binary)
+        service.enable(self, host, binary)
 
     def services(self):
         """Return a generator of service
@@ -1150,7 +1262,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype:
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._create(_volume_attachment.VolumeAttachment,
                             server_id=server_id, **attrs)
 
@@ -1177,7 +1289,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(volume_attachment, server,
                                             "server_id")
-        volume_attachment = resource2.Resource._get_id(volume_attachment)
+        volume_attachment = resource.Resource._get_id(volume_attachment)
 
         return self._update(_volume_attachment.VolumeAttachment,
                             attachment_id=volume_attachment,
@@ -1206,7 +1318,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(volume_attachment, server,
                                             "server_id")
-        volume_attachment = resource2.Resource._get_id(volume_attachment)
+        volume_attachment = resource.Resource._get_id(volume_attachment)
 
         self._delete(_volume_attachment.VolumeAttachment,
                      attachment_id=volume_attachment,
@@ -1239,7 +1351,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(volume_attachment, server,
                                             "server_id")
-        volume_attachment = resource2.Resource._get_id(volume_attachment)
+        volume_attachment = resource.Resource._get_id(volume_attachment)
 
         return self._get(_volume_attachment.VolumeAttachment,
                          server_id=server_id,
@@ -1256,7 +1368,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype:
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._list(_volume_attachment.VolumeAttachment, paginated=False,
                           server_id=server_id)
 
@@ -1268,17 +1380,48 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.migrate(self._session)
+        server.migrate(self)
 
-    def live_migrate_server(self, server, host=None, force=False):
-        """Migrate a server from one host to target host
+    def live_migrate_server(
+            self, server, host=None, force=False, block_migration=None):
+        """Live migrate a server from one host to target host
 
-        :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-        :param host: The host to which to migrate the server
-        :param force: Force a live-migration by not verifying the provided
-                      destination host by the scheduler.
+        :param server:
+            Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param str host:
+            The host to which to migrate the server. If the Nova service is
+            too old, the host parameter implies force=True which causes the
+            Nova scheduler to be bypassed. On such clouds, a ``ValueError``
+            will be thrown if ``host`` is given without ``force``.
+        :param bool force:
+            Force a live-migration by not verifying the provided destination
+            host by the scheduler. This is unsafe and not recommended.
+        :param block_migration:
+            Perform a block live migration to the destination host by the
+            scheduler.  Can be 'auto', True or False. Some clouds are too old
+            to support 'auto', in which case a ValueError will be thrown. If
+            omitted, the value will be 'auto' on clouds that support it, and
+            False on clouds that do not.
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.live_migrate(self._session, host, force)
+        server.live_migrate(
+            self, host,
+            force=force,
+            block_migration=block_migration)
+
+    def wait_for_delete(self, res, interval=2, wait=120):
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait)
