@@ -15,7 +15,6 @@
 import copy
 import warnings
 
-from keystoneauth1 import adapter
 from keystoneauth1 import discover
 import keystoneauth1.exceptions.catalog
 from keystoneauth1 import session as ks_session
@@ -23,6 +22,7 @@ import os_service_types
 import requestsexceptions
 from six.moves import urllib
 
+from openstack import _adapter
 from openstack import version as openstack_version
 from openstack import _log
 from openstack.config import _util
@@ -174,11 +174,16 @@ class CloudRegion(object):
 
     def get_requests_verify_args(self):
         """Return the verify and cert values for the requests library."""
-        if self.config.get('verify') and self.config.get('cacert'):
-            verify = self.config.get('cacert')
+        insecure = self.config.get('insecure', False)
+        verify = self.config.get('verify', True)
+        cacert = self.config.get('cacert')
+        # Insecure is the most aggressive setting, so it wins
+        if insecure:
+            verify = False
+        if verify and cacert:
+            verify = cacert
         else:
-            verify = self.config.get('verify')
-            if self.config.get('cacert'):
+            if cacert:
                 warnings.warn(
                     "You are specifying a cacert for the cloud {full_name}"
                     " but also to ignore the host verification. The host SSL"
@@ -207,7 +212,8 @@ class CloudRegion(object):
     def _get_config(
             self, key, service_type,
             default=None,
-            fallback_to_unprefixed=False):
+            fallback_to_unprefixed=False,
+            converter=None):
         '''Get a config value for a service_type.
 
         Finds the config value for a key, looking first for it prefixed by
@@ -223,20 +229,52 @@ class CloudRegion(object):
 
         If none of that works, it returns the value in ``default``.
         '''
+        if service_type is None:
+            return self.config.get(key)
+
         for st in self._service_type_manager.get_all_types(service_type):
             value = self.config.get(_make_key(key, st))
             if value is not None:
-                return value
-        if fallback_to_unprefixed:
-            return self.config.get(key)
-        return default
+                break
+        else:
+            if fallback_to_unprefixed:
+                value = self.config.get(key)
+
+        if value is None:
+            return default
+        else:
+            if converter is not None:
+                value = converter(value)
+            return value
+
+    def _get_service_config(self, key, service_type):
+        config_dict = self.config.get(key)
+        if not config_dict:
+            return None
+        if not isinstance(config_dict, dict):
+            return config_dict
+
+        for st in self._service_type_manager.get_all_types(service_type):
+            if st in config_dict:
+                return config_dict[st]
 
     def get_interface(self, service_type=None):
         return self._get_config(
             'interface', service_type, fallback_to_unprefixed=True)
 
     def get_api_version(self, service_type):
-        return self._get_config('api_version', service_type)
+        version = self._get_config('api_version', service_type)
+        if version:
+            try:
+                float(version)
+            except ValueError:
+                if 'latest' in version:
+                    warnings.warn(
+                        "You have a configured API_VERSION with 'latest' in"
+                        " it. In the context of openstacksdk this doesn't make"
+                        " any sense.")
+                return None
+        return version
 
     def get_default_microversion(self, service_type):
         return self._get_config('default_microversion', service_type)
@@ -262,8 +300,23 @@ class CloudRegion(object):
         value = self._get_config('endpoint_override', service_type)
         if not value:
             value = self._get_config('endpoint', service_type)
+        if not value and self.config.get('auth_type') == 'none':
+            # If endpoint is given and we're using the none auth type,
+            # then the endpoint value is the endpoint_override for every
+            # service.
+            value = self.config.get('auth', {}).get('endpoint')
         return value
 
+    def get_connect_retries(self, service_type):
+        return self._get_config('connect_retries', service_type,
+                                fallback_to_unprefixed=True,
+                                converter=int)
+
+    def get_status_code_retries(self, service_type):
+        return self._get_config('status_code_retries', service_type,
+                                fallback_to_unprefixed=True,
+                                converter=int)
+
     @property
     def prefer_ipv6(self):
         return not self._force_ipv4
@@ -381,8 +434,23 @@ class CloudRegion(object):
 
         return version_request
 
+    def get_all_version_data(self, service_type):
+        # Seriously. Don't think about the existential crisis
+        # that is the next line. You'll wind up in cthulhu's lair.
+        service_type = self.get_service_type(service_type)
+        versions = self.get_session().get_all_version_data(
+            service_type=service_type,
+            interface=self.get_interface(service_type),
+            region_name=self.region_name,
+        )
+        region_versions = versions.get(self.region_name, {})
+        interface_versions = region_versions.get(
+            self.get_interface(service_type), {})
+        return interface_versions.get(service_type, [])
+
     def get_session_client(
-            self, service_type, version=None, constructor=adapter.Adapter,
+            self, service_type, version=None,
+            constructor=_adapter.OpenStackSDKAdapter,
             **kwargs):
         """Return a prepped keystoneauth Adapter for a given service.
 
@@ -399,17 +467,51 @@ class CloudRegion(object):
         """
         version_request = self._get_version_request(service_type, version)
 
+        kwargs.setdefault('connect_retries',
+                          self.get_connect_retries(service_type))
+        kwargs.setdefault('status_code_retries',
+                          self.get_status_code_retries(service_type))
+        endpoint_override = self.get_endpoint(service_type)
+        version = version_request.version
+        min_api_version = (
+            kwargs.pop('min_version', None) or version_request.min_api_version)
+        max_api_version = (
+            kwargs.pop('max_version', None) or version_request.max_api_version)
+        # Older neutron has inaccessible discovery document. Nobody noticed
+        # because neutronclient hard-codes an append of v2.0. YAY!
+        if service_type == 'network':
+            version = None
+            min_api_version = None
+            max_api_version = None
+            if endpoint_override is None:
+                network_adapter = constructor(
+                    session=self.get_session(),
+                    service_type=self.get_service_type(service_type),
+                    service_name=self.get_service_name(service_type),
+                    interface=self.get_interface(service_type),
+                    region_name=self.region_name,
+                )
+                network_endpoint = network_adapter.get_endpoint()
+                if not network_endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
+                    if not network_endpoint.endswith('/'):
+                        network_endpoint += '/'
+                    network_endpoint = urllib.parse.urljoin(
+                        network_endpoint, 'v2.0')
+                endpoint_override = network_endpoint
+
         client = constructor(
             session=self.get_session(),
             service_type=self.get_service_type(service_type),
             service_name=self.get_service_name(service_type),
             interface=self.get_interface(service_type),
             region_name=self.region_name,
-            version=version_request.version,
-            min_version=version_request.min_api_version,
-            max_version=version_request.max_api_version,
-            endpoint_override=self.get_endpoint(service_type),
+            version=version,
+            min_version=min_api_version,
+            max_version=max_api_version,
+            endpoint_override=endpoint_override,
             default_microversion=version_request.default_microversion,
+            rate_limit=self.get_rate_limit(service_type),
+            concurrency=self.get_concurrency(service_type),
             **kwargs)
         if version_request.default_microversion:
             default_microversion = version_request.default_microversion
@@ -636,3 +738,11 @@ class CloudRegion(object):
 
     def get_password_callback(self):
         return self._password_callback
+
+    def get_rate_limit(self, service_type=None):
+        return self._get_service_config(
+            'rate_limit', service_type=service_type)
+
+    def get_concurrency(self, service_type=None):
+        return self._get_service_config(
+            'concurrency', service_type=service_type)
