@@ -14,7 +14,6 @@
 import copy
 
 from openstack import exceptions
-from openstack.object_store import object_store_service
 from openstack.object_store.v1 import _base
 from openstack import resource
 
@@ -32,11 +31,10 @@ class Object(_base.BaseResource):
 
     base_path = "/%(container)s"
     pagination_key = 'X-Container-Object-Count'
-    service = object_store_service.ObjectStoreService()
 
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
     allow_head = True
