@@ -20,6 +20,7 @@ from openstack.orchestration.v1 import stack_files as _stack_files
 from openstack.orchestration.v1 import stack_template as _stack_template
 from openstack.orchestration.v1 import template as _template
 from openstack import proxy
+from openstack import resource
 
 
 class Proxy(proxy.Proxy):
@@ -59,7 +60,7 @@ class Proxy(proxy.Proxy):
     def stacks(self, **query):
         """Return a generator of stacks
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of stack objects
@@ -84,7 +85,7 @@ class Proxy(proxy.Proxy):
 
         :param stack: The value can be the ID of a stack or a
                :class:`~openstack.orchestration.v1.stack.Stack` instance.
-        :param kwargs \*\*attrs: The attributes to update on the stack
+        :param kwargs attrs: The attributes to update on the stack
                                  represented by ``value``.
 
         :returns: The updated stack
@@ -184,14 +185,14 @@ class Proxy(proxy.Proxy):
             stk = self._find(_stack.Stack, stack, ignore_missing=False)
 
         obj = _stack_files.StackFiles(stack_name=stk.name, stack_id=stk.id)
-        return obj.get(self)
+        return obj.fetch(self)
 
     def resources(self, stack, **query):
         """Return a generator of resources
 
         :param stack: This can be a stack object, or the name of a stack
                       for which the resources are to be listed.
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of resource objects if the stack exists and
@@ -360,3 +361,44 @@ class Proxy(proxy.Proxy):
         return tmpl.validate(self, template, environment=environment,
                              template_url=template_url,
                              ignore_errors=ignore_errors)
+
+    def wait_for_status(self, res, status='ACTIVE', failures=None,
+                        interval=2, wait=120):
+        """Wait for a resource to be in a particular status.
+
+        :param res: The resource to wait on to reach the specified status.
+                    The resource must have a ``status`` attribute.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param status: Desired status.
+        :param failures: Statuses that would be interpreted as failures.
+        :type failures: :py:class:`list`
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to the desired status failed to occur in specified seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+                 has transited to one of the failure statuses.
+        :raises: :class:`~AttributeError` if the resource does not have a
+                ``status`` attribute.
+        """
+        failures = [] if failures is None else failures
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait)
+
+    def wait_for_delete(self, res, interval=2, wait=120):
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait)
