@@ -61,14 +61,18 @@ class TestPort(base.TestCase):
         self.assertEqual('port', sot.resource_key)
         self.assertEqual('ports', sot.resources_key)
         self.assertEqual('/ports', sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
-        self.assertDictEqual({"description": "description",
+        self.assertDictEqual({"binding:host_id": "binding:host_id",
+                              "binding:profile": "binding:profile",
+                              "binding:vif_details": "binding:vif_details",
+                              "binding:vif_type": "binding:vif_type",
+                              "binding:vnic_type": "binding:vnic_type",
+                              "description": "description",
                               "device_id": "device_id",
                               "device_owner": "device_owner",
                               "fixed_ips": "fixed_ips",
